<?php

/*

    GTPhoto2 - A multi user photo album
    Copyright (C) 2004-2006 Chris Kloosterman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/

	# email daemon

	# only one email daemon may run at once.  It is not multi-process safe.
	# to make it multi-process safe, add appropriate lock table and unlock table statements

	require_once ("./framework/includes.php");

	# do not cache sql results
	define ('DO_CACHING_OVERRIDE', 1);

	# define SQL here
	$field_array = array("hidden");
	$hide_stmt = sql_build_manip("rss_item", $field_array, MDB2_AUTOQUERY_UPDATE, "date < ?");
	$delete_stmt = sql_build_manip("rss_item", NULL, MDB2_AUTOQUERY_DELETE, "date < ?");
	$field_array = array("rss_item_id", "rss_id", "title", "summary", "url", "date");
	$insert_stmt = sql_build_manip("rss_item", $field_array, MDB2_AUTOQUERY_INSERT, NULL);

	while (1) {

		# figure out which RSS feeds to download
		$SQL = "SELECT rss.rss_id, url
			FROM rss
			INNER JOIN rss_subscription ON (rss.rss_id = rss_subscription.rss_id)
			GROUP BY rss.rss_id, url
			HAVING count(rss_subscription_id) > 0";
		$result = sql_query_read($SQL);

		# go through each subscribed feed and fetch the latest rss
		while ($row = sql_fetch_array($result)) {
			$rss = new RssFeed($row["url"]);
			$items = $rss->getEntries();
			$indiv = explode("|||", $items);
			# insert the new RSS items until we find one
			# that we already have (then abort)
			foreach ($indiv as $key => $value) {
				if ($value == "") {
					continue;
				}
				$values = explode("||", $value);
				$SQL = "SELECT rss_item_id
					FROM rss_item
					WHERE url LIKE ".clean_form_sql($values[1]);
				$result2 = sql_query_read($SQL);
				if (sql_num_rows($result2) > 0) {
					# we've found an entry we already have
					# so skip it and keep searching
					continue;
				} else {
					$id = sql_next_id("rss_item");
					$insert_array = array($id, $row["rss_id"], mb_convert_encoding(html_entity_decode($values[0], ENT_QUOTES),"UTF-8"), mb_convert_encoding(html_entity_decode($values[2], ENT_QUOTES),"UTF-8"), $values[1], time());
					sql_execute_manip($insert_stmt, $insert_array);
					echo ("Inserted.\n");
				}
			}
		}

		# delete the items older than ~ 6 months
		sql_execute_manip($delete_stmt, array(time() - 60*60*24*7*26));
		# mark items > 2 weeks old as hidden
		sql_execute_manip($hide_stmt, array(1, time() - 60*60*24*14));
	
		# wait an hour before starting the whole thing over again
		sleep(3600);
	}

?>
