<?php

/*

    Gavintech Framework - A base for PHP projects
    Copyright (C) 2004-2008 Chris Kloosterman

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/

	# constants
	$text_size = 8;
	$small_text_size = 7;
	$title_size = 12;

	require_once("framework/includes.php");

	$graph = $_SESSION["graph"][$_REQUEST["id"]];

	if (!is_array($graph)) {
		puke("Error loading graph data");
	}

	# determine the overall image size (x direction)
	$x_size = $graph["metadata"]["xsize"];

	# left side

	## figure out the largest y-axis label width
	$size = imagettfbbox_t($text_size, $angle, GD_FONT, $graph["metadata"]["ymax"]);
	$max_left = $size[4] - $size[0];
	
	# gridlines and y axis
	$gridline_height = $graph["metadata"]["ysize"] / ($graph["metadata"]["show_gridlines"] + 1);
	$gridline_adder = ($graph["metadata"]["ymax"] - $graph["metadata"]["ymin"]) / ($graph["metadata"]["show_gridlines"] + 1);
	
	# the gridline labels
	for ($i = 1; $i <= $graph["metadata"]["show_gridlines"]; $i++) {
		$y_value = $graph["metadata"]["ymax"] - ($gridline_adder * $i);
		$y_value = round($y_value * 100) / 100;
		$size = imagettfbbox_t($text_size, $angle, GD_FONT, $y_value);
		if ($size[4] - $size[0] > $max_left) {
			$max_left = $size[4] - $size[0];
		}
	}
	
	# the bottom axis label
	$size = imagettfbbox_t($text_size, $angle, GD_FONT, $graph["metadata"]["ymin"]);
	if ($size[4] - $size[0] > $max_left) {
		$max_left = $size[4] - $size[0];
	}

	if ($graph["metadata"]["y_axis_legend"]) {
		$y_legend_size = $text_size + 4;
	}

	if ($graph["metadata"]["y_axis_units"]) {
		$y_units_size = $small_text_size + 4;
	}

	#### can't actually set $left yet, we need to figure out whether the first label will overflow to the right

	# right side
	$right = 20;

	$x_size += $right;
	
	$y_size = $graph["metadata"]["ysize"];
	# top
	$top = 0;
	## space
	$top += 5;
	## title
	$top += $title_size;
	## space
	$top += 8;

	$y_size += $top;

	# bottom
	$bottom = 0;
	## space
	$bottom += 5;
	## legend vs. timing stats
	$legend = 0;
	$timing = 0;
	if ($graph["metadata"]["show_legend"]) {
		foreach ($graph as $key => $value) {
			if ($graph[$key]["legend"]) {
				$legend += $text_size + 5;
			}
		}
	}
	if ($graph["metadata"]["show_proctime"]) {
		$timing += $text_size + 2;
	}
	if ($graph["metadata"]["show_gentime"]) {
		$timing += $text_size + 2;
	}

	if ($legend > $timing) {
		$bottom += $legend;
	} else {
		$bottom += $timing;
	}
	## space
	$bottom += 8;

	## x labels if there are any
	$x_labels = 0;
	if ($graph["metadata"]["show_x_labels"]) {
		if ($graph["metadata"]["x_label_angle"]) {
			$angle = $graph["metadata"]["x_label_angle"];
		} else {
			$angle = 45;
		}
		foreach ($graph[0]["data"] as $key => $value) {
			# if there's no key, assume we're counting one at a time
			if (!isset($key)) {
				$key = ++$count;
			}
			$size = imagettfbbox_t($text_size, $angle, GD_FONT, $key);

			# this may be wider than all of the y axis labels
			if (!isset($first_x_label_width)) {
				$first_x_label_width = $size[2] - $size[6];
			}

			# figure out whether the height of this label
			# is bigger than all of the rest so far
			if ($size[1] - $size[5] > $x_labels) {
				$x_labels = $size[1] - $size[5];
			}
		}
	}

	if ($x_labels > 0) {
		# additional spacing between graph and label text
		$x_labels += 4;
	}

	$bottom += $x_labels;

	#### this is where we set $left
	if ($first_x_label_width > ($max_left + $y_legend_size + $y_units_size + 4)) {
		$left = $first_x_label_width;
	} else {
		$left = $max_left + $y_legend_size + $y_units_size + 4;
	}
	$x_size += $left;

	# x axis legend
	if ($graph["metadata"]["x_axis_legend"]) {
		$x_legend_size = $text_size + 4;
		$bottom += $x_legend_size;
	}

	# x axis units
	if ($graph["metadata"]["x_axis_units"]) {
		$x_units_size += $small_text_size + 4;
		$bottom += $x_units_size;
	}

	$y_size += $bottom;

	# special calculations only for bar graphs
	if ($graph["metadata"]["type"] == "bar") {
		if ($graph["metadata"]["bar_gap"]) {
			$bar_gap = $graph["metadata"]["bar_gap"];
		} else {
			$bar_gap = 8;
		}
		if ($graph["metadata"]["multiple_bar_offset"]) {
			$multiple_bar_offset = $graph["metadata"]["multiple_bar_offset"];
		} else {
			$multiple_bar_offset = 8;
		}

		# figure out how many sets of bars we'll have
		foreach ($graph as $key => $value) {
			if (is_numeric($key)) {
				$count_sets++;
			}
		}
		$number_of_points = count($graph[0]["data"]);
		$number_of_gaps = $number_of_points - 1;
		$number_of_extra_bars = $count_sets - 1;
		$used_width = $number_of_gaps * $bar_gap + $number_of_extra_bars * $multiple_bar_offset * $number_of_points;
		# figure out how wide to make each bar
		$bar_width = ($graph["metadata"]["xsize"] - $used_width) / $number_of_points;
	}

	# special calculations for x_axis_type scaled
	if ($graph["metadata"]["x_axis_type"] == "scaled") {
		# figure out the maximum and minimum x values
		$x_max_val = -1 * (PHP_INT_MAX - 1);
		$x_min_val = PHP_INT_MAX;
		foreach ($graph[0]["data"] as $x_val => $y_val) {
			if ($x_val < $x_min_val) {
				$x_min_val = $x_val;
			}
			if ($x_val > $x_max_val) {
				$x_max_val = $x_val;
			}
		}
	}

	# make the image we're going to use
	$g = imagecreatetruecolor($x_size, $y_size);

	# set the image parameters
	imageantialias($g, true);

	# set up the colours
	foreach ($graph as $key => $value) {
		if (is_numeric($key)) {
			$colour[$key] = imagecolorallocate($g, $value["colour"]["r"], $value["colour"]["g"], $value["colour"]["b"]);
		}
	}
	$colour["line"] = imagecolorallocate($g, 0, 0, 0);
	$colour["text"] = imagecolorallocate($g, 0, 0, 0);
	$colour["white"] = imagecolorallocate($g, 255, 255, 255);
	$colour["bg"] = imagecolorallocate($g, $graph["metadata"]["bgcolour"]["r"], $graph["metadata"]["bgcolour"]["g"], $graph["metadata"]["bgcolour"]["b"]);

	# draw the basic graph structure
	
	## white background
	imagefill($g, 0, 0, $colour["white"]);
	
	## graph border
	imageline($g, $left, $top, $left, $y_size - $bottom, $colour["line"]);
	imageline($g, $left, $top, $x_size - $right, $top, $colour["line"]);
	imageline($g, $left, $y_size - $bottom, $x_size - $right, $y_size - $bottom, $colour["line"]);
	imageline($g, $x_size - $right, $top, $x_size - $right, $y_size - $bottom, $colour["line"]);
	
	## graph backdrop
	imagefill($g, $left + 1, $top + 1, $colour["bg"]);

	## title
	### determine width of text
	$bbox = imagettfbbox($title_size, 0, GD_FONT, $graph["metadata"]["title"]);
	$text_width = $bbox[2] - $bbox[0];
	### determine the offset from the left side of the bounding box
	$text_offset = ($graph["metadata"]["xsize"] - $text_width) / 2;
	### draw the title
	imagettftext($g, $title_size, 0, $left + $text_offset, 5 + $title_size, $colour["text"], GD_FONT, $graph["metadata"]["title"]);

	# draw the gridlines
	for ($i = 1; $i <= $graph["metadata"]["show_gridlines"]; $i++) {
		if ($graph["metadata"]["show_gridlines"] > 0) {
			imageline($g, $left, $top + $i * $gridline_height, $x_size - $right, $top + $i * $gridline_height, $colour["line"]);
		}
	}

	# draw the graphs
	foreach ($graph as $key => $value) {
		if (is_numeric($key)) {
			# this is a series to plot
			$x_values = count($value["data"]);
			$x_width = $graph["metadata"]["xsize"] / ($x_values - 1);
			$y_range = $graph["metadata"]["ymax"] - $graph["metadata"]["ymin"];

			if ($graph["metadata"]["type"] == "bar") {
				$start = 0;
			} else {
				# need to start the graph somewhere, so do some precalculations to figure out where on the graph to start from
				$label = key($value["data"]);
				$data = current($value["data"]);

				$start = 1;
				if ($graph["metadata"]["x_axis_type"] == "scaled") {
					$prev_x = $left + (($label - $x_min_val) / ($x_max_val - $x_min_val)) * $graph["metadata"]["xsize"];
				} else {
					$prev_x = $left;
				}
				$prev_y = ((1 - ($data - $graph["metadata"]["ymin"]) / $y_range) * $graph["metadata"]["ysize"]) + $top;

				# draw the X axis label
				if ($graph["metadata"]["show_x_labels"] && $key == 0) {
					$size = imagettfbbox_t($text_size, $angle, GD_FONT, $label);
					imagettftext($g, $text_size, $angle, $prev_x - $size[2] + 0.5 * $text_size, $y_size - $bottom + 4 - ($size[5] - $size[1]), $colour["text"], GD_FONT, $label);
				}
			}

			# plot each point
			for ($i = $start; $i < $x_values; $i++) {
				if ($graph["metadata"]["type"] != "bar") {
					next($value["data"]);
				}
				$label = key($value["data"]);
				$data = current($value["data"]);

				$new_y = ((1 - ($data - $graph["metadata"]["ymin"]) / $y_range) * $graph["metadata"]["ysize"]) + $top;

				if ($graph["metadata"]["type"] == "bar") {
					$top_x1 = $left + $i * ($bar_width + ($multiple_bar_offset * $number_of_extra_bars) + $bar_gap) + $key * $multiple_bar_offset;
					$top_x2 = $top_x1 + $bar_width;
					$new_x = $top_x1 + ((($multiple_bar_offset * $number_of_extra_bars) + $bar_width) / 2);

					imagefilledrectangle($g, $top_x1, $new_y, $top_x2, $top + $graph["metadata"]["ysize"], $colour[$key]);

					imageline($g, $top_x1, $new_y, $top_x2, $new_y, $colour["line"]);
					imageline($g, $top_x1, $new_y, $top_x1, $top + $graph["metadata"]["ysize"], $colour["line"]);
					imageline($g, $top_x2, $new_y, $top_x2, $top + $graph["metadata"]["ysize"], $colour["line"]);
				} else {
					if ($graph["metadata"]["x_axis_type"] == "scaled") {
						$new_x = $left + (($label - $x_min_val) / ($x_max_val - $x_min_val)) * $graph["metadata"]["xsize"];
					} else {
						$new_x = $left + ($x_width * $i);
					}

					imageline($g, $prev_x, $prev_y, $new_x, $new_y, $colour[$key]);

					$prev_x = $new_x;
					$prev_y = $new_y;
				}

				# draw the X axis label
				if ($graph["metadata"]["show_x_labels"] && $key == 0) {
					$size = imagettfbbox_t($text_size, $angle, GD_FONT, $label);
					imagettftext($g, $text_size, $angle, $new_x - $size[2] + 0.5 * $text_size, $y_size - $bottom + 4 - ($size[5] - $size[1]), $colour["text"], GD_FONT, $label);
				}

				if ($graph["metadata"]["type"] == "bar") {
					next($value["data"]);
				}
			}
		}
	}
	
	# finish off the structure

	## x legend
	if ($graph["metadata"]["x_axis_legend"]) {
		### determine width of text
		$bbox = imagettfbbox($text_size, 0, GD_FONT, $graph["metadata"]["x_axis_legend"]);
		$text_width = $bbox[2] - $bbox[0];
		### determine the offset from the left side of the bounding box
		$text_offset = ($graph["metadata"]["xsize"] - $text_width) / 2;
		imagettftext($g, $text_size, 0, $left + $text_offset, $y_size - $bottom + 4 + $x_labels + $text_size, $colour["text"], GD_FONT, $graph["metadata"]["x_axis_legend"]);
	}

	## x units
	if ($graph["metadata"]["x_axis_units"]) {
		### determine width of text
		$bbox = imagettfbbox($small_text_size, 0, GD_FONT, $graph["metadata"]["x_axis_units"]);
		$text_width = $bbox[2] - $bbox[0];
		### determine the offset from the left side of the bounding box
		$text_offset = ($graph["metadata"]["xsize"] - $text_width) / 2;
		imagettftext($g, $small_text_size, 0, $left + $text_offset, $y_size - $bottom + $x_legend_size + 4 + $x_labels + $small_text_size, $colour["text"], GD_FONT, $graph["metadata"]["x_axis_units"]);
	}

	## y legend
	if ($graph["metadata"]["y_axis_legend"]) {
		### determine height of text
		$bbox = imagettfbbox_t($text_size, 270, GD_FONT, $graph["metadata"]["y_axis_legend"]);
		$text_height = $bbox[5] - $bbox[1];
		### determine the offset from the left side of the bounding box
		$text_offset = ($graph["metadata"]["ysize"] - $text_height) / 2;
		imagettftext($g, $text_size, 270, $y_units_size, $top + $text_offset, $colour["text"], GD_FONT, $graph["metadata"]["y_axis_legend"]);
	}

	## y units
	if ($graph["metadata"]["y_axis_units"]) {
		### determine height of text
		$bbox = imagettfbbox_t($small_text_size, 270, GD_FONT, $graph["metadata"]["y_axis_units"]);
		$text_height = $bbox[5] - $bbox[1];
		### determine the offset from the left side of the bounding box
		$text_offset = ($graph["metadata"]["ysize"] - $text_height) / 2;
		imagettftext($g, $small_text_size, 270, 0, $top + $text_offset, $colour["text"], GD_FONT, $graph["metadata"]["y_axis_units"]);
	}

	## legend
	if ($graph["metadata"]["show_legend"]) {
	
		# determine the maximum text width
		$max_legend = 0;
		foreach ($graph as $key => $value) {
			if (is_numeric($key)) {
				$bbox = imagettfbbox($text_size, 0, GD_FONT, $value["legend"]);
				$legend_width = $bbox[2] - $bbox[0];
				if ($legend_width > $max_legend) {
					$max_legend = $legend_width;
				}
			}
		}
		
		# print the legends
		$count = 0;
		foreach ($graph as $key => $value) {
			if (is_numeric($key) && isset($value["legend"])) {
				imagettftext($g, $text_size, 0, $x_size - $right - $max_legend, $y_size - $bottom + 8 + $x_labels + $x_legend_size + $x_units_size + $count * ($text_size + 5) + $text_size, $colour["text"], GD_FONT, $value["legend"]);
				imageline($g, $x_size - $right - $max_legend - 20, $y_size - $bottom + 8 + $x_labels + $x_legend_size + $x_units_size + $count * ($text_size + 5) + 0.5 * $text_size, $x_size - $right - $max_legend - 5, $y_size - $bottom + 8 + $x_labels + $x_legend_size + $x_units_size + $count * ($text_size + 5) + 0.5 * $text_size, $colour[$key]);
				$count++;
			}
		}
		
	}

	# the top axis label
	imagettftext($g, $text_size, 0, $y_units_size + $y_legend_size, $top + ($text_size / 2), $colour["text"], GD_FONT, $graph["metadata"]["ymax"]);
	
	# the gridline labels
	for ($i = 1; $i <= $graph["metadata"]["show_gridlines"]; $i++) {
		$y_value = $graph["metadata"]["ymax"] - ($gridline_adder * $i);
		$y_value = round($y_value * 100) / 100;
		imagettftext($g, $text_size, 0, $y_units_size + $y_legend_size, $top + ($i * $gridline_height) + ($text_size / 2), $colour["text"], GD_FONT, $y_value);
	}
	
	# the bottom axis label
	imagettftext($g, $text_size, 0, $y_units_size + $y_legend_size, $top + ($gridline_height * ($graph["metadata"]["show_gridlines"] + 1)) + ($text_size / 2), $colour["text"], GD_FONT, $graph["metadata"]["ymin"]);

	## redo the graph border to fix where it has been
	## slightly overdrawn by the graph lines
	imageline($g, $left, $top, $left, $y_size - $bottom, $colour["line"]);
	imageline($g, $left, $top, $x_size - $right, $top, $colour["line"]);
	imageline($g, $left, $y_size - $bottom, $x_size - $right, $y_size - $bottom, $colour["line"]);
	imageline($g, $x_size - $right, $top, $x_size - $right, $y_size - $bottom, $colour["line"]);

	## generation time, processing time
	if ($graph["metadata"]["show_gentime"]) {
		$time = date("F j, Y H:i T");
		imagettftext($g, $text_size, 0, $left, $y_size - $bottom + 8 + $x_labels + $x_legend_size + $x_units_size + $text_size, $colour["text"], GD_FONT, "Generated $time");
		$offset_add = $text_size + 5;
	}
	
	if ($graph["metadata"]["show_proctime"]) {
		global $PAGE_START_TIME;
		$PAGE_END_TIME = get_microtime();
		$PAGE_DRAW_TIME = $PAGE_END_TIME - $PAGE_START_TIME;
		
		$time = round($PAGE_DRAW_TIME * 1000) / 1000;
		
		imagettftext($g, $text_size, 0, $left, $y_size - $bottom + 8 + $x_labels + $x_legend_size + $x_units_size + $text_size + $offset_add, $colour["text"], GD_FONT, "Processing time: $time seconds");
	}
	
	# output the image
	header("Content-Type: image/png");
	imagepng($g);

	# now get rid of the image we generated
	imagedestroy($g);

# http://ruquay.com/sandbox/imagettf/
function imagettfbbox_t($size, $angle, $fontfile, $text){
	// compute size with a zero angle
	$coords = imagettfbbox($size, 0, $fontfile, $text);
	// convert angle to radians
	$a = deg2rad($angle);
	// compute some usefull values
	$ca = cos($a);
	$sa = sin($a);
	$ret = array();
	// perform transformations
	for($i = 0; $i < 7; $i += 2){
		$ret[$i] = round($coords[$i] * $ca + $coords[$i+1] * $sa);
		$ret[$i+1] = round($coords[$i+1] * $ca - $coords[$i] * $sa);
	}
	return $ret;
}

?>
