<?php
/*

    Gavintech Framework - A base for PHP projects
    Copyright (C) 2004-2008 Chris Kloosterman

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/

require_login(AUTH_USER);

# cart operations

$done = array();

# build SQL statements
$cart_stmt = sql_build_manip("cart", NULL, MDB2_AUTOQUERY_DELETE, "cart_id = ?");

if (is_array($_REQUEST["delete"]) && count($errors) == 0) {
	require_validator("cart.php");
	
	foreach ($_REQUEST["delete"] as $key => $value) {	
		# get the item to delete
		$SQL = "SELECT name, key_row, id FROM cart INNER JOIN cart_table ON (cart.table_id = cart_table.table_id) WHERE cart_id = ".clean_form_sql($key);
		$result = sql_query_read($SQL);
		
		if (sql_num_rows($result) > 0) {
			$row = sql_fetch_array($result);

			$delete_stmt = sql_build_manip($row["name"], NULL, MDB2_AUTOQUERY_DELETE, $row["key_row"]." = ? AND is_ordered = 0");
			sql_execute_manip($delete_stmt, array($row["id"]));
			sql_execute_manip($cart_stmt, array($key));
			array_push($done, "Deleted item");
		}
	}
}

if ($_REQUEST["clear"]) {
	# get all items in the cart
	$SQL = "SELECT name, cart_id, key_row, id FROM cart INNER JOIN cart_table ON (cart.table_id = cart_table.table_id) WHERE user_id = ".clean_form_sql($_SESSION["user_id"]);
	$result = sql_query_read($SQL);
	
	# if there are items, kill them
	if (sql_num_rows($result) > 0) {
		while ($row = sql_fetch_array($result)) {
			# also kill the description in the extra info table
			$delete_stmt = sql_build_manip($row["name"], NULL, MDB2_AUTOQUERY_DELETE, $row["key_row"]." = ? AND is_ordered = 0");
			sql_execute_manip($delete_stmt, array($row["id"]));
			sql_execute_manip($cart_stmt, array($row["cart_id"]));
		}
		array_push($done, "Cleared cart");
	}
}

if ($_REQUEST["update"] && count($errors) == 0) {
	# get all items in the cart
	$SQL = "SELECT cart_id, name, key_row, id FROM cart INNER JOIN cart_table ON (cart.table_id = cart_table.table_id) WHERE user_id = ".clean_form_sql($_SESSION["user_id"]);
	$result = sql_query_read($SQL);
	
	# if there are items, update them
	if (sql_num_rows($result) > 0) {
		while ($row = sql_fetch_array($result)) {
			$field_array = array("quantity");
			$stmt = sql_build_manip($row["name"], $field_array, MDB2_AUTOQUERY_UPDATE, $row["key_row"]." = ?");
			$update_array = array($_REQUEST["quantity"][$row["cart_id"]], $row["id"]);
			sql_execute_manip($stmt, $update_array);
		}
		array_push($done, "Updated cart");
	}
}

?>

<h1>Shopping cart</h1>

<?php

print_errors();

# print messages if we accomplished anything
if (count($done) > 0) {
	echo ("<h2>Success</h2>\n");
	echo ("<p>The following actions were performed</p>\n<ul>\n");
	foreach ($done as $key => $value) {
		echo ("<li>$value</li>");
	}
	echo ("</ul>\n");
}

# get the items in the cart
$SQL = "SELECT cart_id, id, name, key_row, details_page
	FROM cart
	INNER JOIN cart_table ON (cart.table_id = cart_table.table_id)
	WHERE user_id = ".clean_form_sql($_SESSION["user_id"]);
$result = sql_query_read($SQL);

$array = array();

$total = 0;

# paginate them
while ($row = sql_fetch_array($result)) {
	# we need specific info from the corresponding table
	$SQL = "SELECT price, name, quantity
		FROM ".$row["name"]."
		WHERE ".$row["key_row"]."=".clean_sql_sql($row["id"]);
	$result2 = sql_query_read($SQL);
	$row2 = sql_fetch_array($result2);

	$array2 = array();
	$array2["Name"] = clean_sql_display($row2["name"]);
	$array2["Unit Price"][0] = clean_sql_display($row2["price"]);
	$array2["Unit Price"][1] = "$".clean_sql_display($row2["price"]);
	$array2["Quantity"][0] = $row2["quantity"];
	$array2["Quantity"][1] = "<input type=\"text\" name=\"quantity[".$row["cart_id"]."]\" value=\"".clean_sql_form($row2["quantity"])."\" />";
	$price = $row2["price"] * $row2["quantity"];
	$array2["Price"][0] = $price;
	$array2["Price"][1] = "$".number_format($price, 2, ".", "");
	$array2[1] = "<a href=\"index.php?next_page=".$row["details_page"]."&amp;".$row["key_row"]."=".$row["id"]."\">Details</a>";
	$array2[2] = "<input type=\"submit\" name=\"delete[".$row["cart_id"]."]\" value=\"Delete\" />";

	$array[] = $array2;

	$total += $price;

}

?>

<form action="index.php" method="post">
<input type="hidden" name="next_page" value="cart.php" />
<input type="hidden" name="validator" value="cart.php" />

<?php

if (count($array) > 0) {
	paginate($array);

?>

<p>Total: $<?php echo(number_format($total, 2, ".", "")); ?></p>

<table width="100%">
<tr>
<td width="33%" align="left">
<input type="submit" name="update" value="Update cart" />
</form>
</td>
<td width="34%" align="center">
<form action="index.php" method="post">
<input type="hidden" name="next_page" value="cart.php" />
<input type="submit" name="clear" value="Clear cart" />
</form>
</td>
<td width="33%" align="right">

<?php
if (IS_PRODUCTION) {
?>

<form action="https://www.paypal.com/cgi-bin/webscr" method="post">

<input type="hidden" name="cmd" value="_xclick">
<input type="hidden" name="business" value="<?php echo(PAYPAL_ACCOUNT); ?>" />
<input type="hidden" name="item_name" value="<?php echo(FRAMEWORK_NAME); ?> Shopping Cart" />
<input type="hidden" name="currency_code" value="CAD" />
<input type="hidden" name="amount" value="<?php echo($total); ?>" />
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="no_note" value="1" />
<input type="hidden" name="return" value="<?php echo($_SERVER["SERVER_PORT"] == 80 ? "http" : "https"); ?>://<?php echo($_SERVER["SERVER_NAME"]); ?><?php echo($_SERVER["PHP_SELF"]); ?>?next_page=cart_complete.php&item_number=<?php echo($total * 100); ?>" />
<?php
} else {
?>
<form action="index.php" method="post">
<input type="hidden" name="next_page" value="cart_complete.php" />
<?php
}
?>
<input type="hidden" name="item_number" value="<?php echo($total * 100); ?>" />
<input type="submit" name="complete" value="Check out" />
</form>
</td>
</tr>
</table>

<?php
} else {
	print "<p><i>Your shopping cart is empty!</i></p>";      
} # if count($array) > 0
?>

</form>

