<?php
/*

    Gavintech Framework - A base for PHP projects
    Copyright (C) 2004-2008 Chris Kloosterman

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/
?>

<h1>How to Draw a Graph</h1>

<h2>Setting up the array</h2>

<p>The general structure of the graph array is as follows:</p>

<pre>
$graph[0]["data"] = array(values);
$graph[0]["colour"] = array("r" => #, "g" => #, "b" => #);
$graph[0]["legend"] = "Name of the series";
$graph[1]...[n] same as above

$graph["metadata"]["title"] = "Graph title";
$graph["metadata"]["bgcolour"] = array("r" => #, "g" => #, "b" => #);
$graph["metadata"]["xsize"] = (width of the chart);
$graph["metadata"]["ysize"] = (height of the chart);
$graph["metadata"]["ymin"] = (what number to start with at the bottom of the graph);
$graph["metadata"]["ymax"] = (what number to end with at the top of the graph);
$graph["metadata"]["show_legend"] = (0 - don't show, 1 - show);
$graph["metadata"]["show_gridlines"] = (0 - don't show, positive number - the number of gridlines);
$graph["metadata"]["show_proctime"] = (0 - don't show, 1 - show);
$graph["metadata"]["show_gentime"] = (0 don't show, 1 - show);
</pre>

<p>Here's one for bar charts:</p>

<pre>
$graph = array();

$array = array();
$array["data"] = array("21/04/07" => 1, "22/04/07" => 4, "23/04/07" => 9, "24/04/07" => 16, "25/04/07" => 25, "25/04/07" => 36, "26/04/07" => 49, "27/04/07" => 64, "28/04/07" => 81, "29/04/07" => 100);
$array["colour"] = array("r" => 245, "g" => 141, "b" => 124);
$array["legend"] = "Squares";
$graph[] = $array;

$array = array();
$array["data"] = array(1, 1, 2, 3, 5, 8, 13, 21, 34, 55);
$array["colour"] = array("r" => 6, "g" => 167, "b" => 22);
$array["legend"] = "Fibonacci";
$graph[] = $array;

$graph["metadata"]["title"] = "Two Mathematical Series (bar)";
$graph["metadata"]["x_axis_legend"] = "Date";
$graph["metadata"]["x_axis_units"] = "(days)";
$graph["metadata"]["y_axis_legend"] = "Number";
$graph["metadata"]["y_axis_units"] = "(units)";
$graph["metadata"]["bgcolour"] = array("r" => 200, "g" => 200, "b" => 200);
$graph["metadata"]["xsize"] = 500;
$graph["metadata"]["ysize"] = 300;
$graph["metadata"]["ymin"] = 0;
$graph["metadata"]["ymax"] = 100;
$graph["metadata"]["type"] = "bar";
$graph["metadata"]["bar_gap"] = 16;
$graph["metadata"]["multiple_bar_offset"] = 8;
$graph["metadata"]["show_x_labels"] = true;
$graph["metadata"]["show_legend"] = 1;
$graph["metadata"]["show_gridlines"] = 4;
$graph["metadata"]["show_proctime"] = 1;
$graph["metadata"]["show_gentime"] = 1;
$graph["metadata"]["x_label_angle"] = 45;
</pre>

<p>Here's the third graph</p>

<pre>
$graph = array();

$array = array();
$array["data"] = array(1 => 5, 2 => 10, 4 => 15, 8 => 20, 16 => 25, 32 => 30);
$array["colour"] = array("r" => 202, "g" => 167, "b" => 22);
$array["legend"] = "Test1";
$graph[] = $array;

$array = array();
$array["data"] = array(1 => 75, 20 => 75, 10 => 25, 32 => 25);
$array["colour"] = array("r" => 245, "g" => 141, "b" => 124);
$array["legend"] = "Z Test";
$graph[] = $array;

$graph["metadata"]["title"] = "Two Plots";
$graph["metadata"]["bgcolour"] = array("r" => 200, "g" => 200, "b" => 200);
$graph["metadata"]["x_axis_type"] = "scaled";
$graph["metadata"]["xsize"] = 500;
$graph["metadata"]["ysize"] = 300;
$graph["metadata"]["ymin"] = 0;
$graph["metadata"]["ymax"] = 100;
$graph["metadata"]["show_x_labels"] = true;
$graph["metadata"]["show_legend"] = 1;
$graph["metadata"]["show_gridlines"] = 4;
$graph["metadata"]["show_proctime"] = 1;
$graph["metadata"]["show_gentime"] = 1;
</pre>

<h2>Graphing</h2>

<p>To actually draw the graph, call draw_graph($graph).</p>

<h2>Sample</h2>

<p>See the source code for this page to see how the graph was generated.</p>

<?php

$graph = array();

$array = array();
$array["data"] = array(1, 1, 2, 3, 5, 8, 13, 21, 34, 55);
$array["colour"] = array("r" => 202, "g" => 167, "b" => 22);
$array["legend"] = "Fibonacci";
$graph[] = $array;

$array = array();
$array["data"] = array(1, 4, 9, 16, 25, 36, 49, 64, 81, 100);
$array["colour"] = array("r" => 245, "g" => 141, "b" => 124);
$array["legend"] = "Squares";
$graph[] = $array;

$array = array();
$array["data"] = array(1, 8, 28, 56, 70, 56, 28, 8, 1);
$array["colour"] = array("r" => 1, "g" => 61, "b" => 4);
$array["legend"] = "Pascal(9)";
$graph[] = $array;

$graph["metadata"]["title"] = "Three Mathematical Series";
$graph["metadata"]["bgcolour"] = array("r" => 200, "g" => 200, "b" => 200);
$graph["metadata"]["xsize"] = 500;
$graph["metadata"]["ysize"] = 300;
$graph["metadata"]["ymin"] = 0;
$graph["metadata"]["ymax"] = 100;
$graph["metadata"]["show_legend"] = 1;
$graph["metadata"]["show_gridlines"] = 4;
$graph["metadata"]["show_proctime"] = 1;
$graph["metadata"]["show_gentime"] = 1;

echo ("<p>");
draw_graph($graph);
echo ("</p>");

$graph = array();

$array = array();
$array["data"] = array("21/04/07" => 1, "22/04/07" => 4, "23/04/07" => 9, "24/04/07" => 16, "25/04/07" => 25, "26/04/07" => 36, "27/04/07" => 49, "28/04/07" => 64, "29/04/07" => 81, "30/04/07" => 100);
$array["colour"] = array("r" => 245, "g" => 141, "b" => 124);
$array["legend"] = "Squares";
$graph[] = $array;

$array = array();
$array["data"] = array(1, 1, 2, 3, 5, 8, 13, 21, 34, 55);
$array["colour"] = array("r" => 6, "g" => 167, "b" => 22);
$array["legend"] = "Fibonacci";
$graph[] = $array;

$graph["metadata"]["title"] = "Two Mathematical Series (bar)";
$graph["metadata"]["x_axis_legend"] = "Date";
$graph["metadata"]["x_axis_units"] = "(days)";
$graph["metadata"]["y_axis_legend"] = "Number";
$graph["metadata"]["y_axis_units"] = "(units)";
$graph["metadata"]["bgcolour"] = array("r" => 200, "g" => 200, "b" => 200);
$graph["metadata"]["xsize"] = 500;
$graph["metadata"]["ysize"] = 300;
$graph["metadata"]["ymin"] = 0;
$graph["metadata"]["ymax"] = 100;
$graph["metadata"]["type"] = "bar";
$graph["metadata"]["bar_gap"] = 16;
$graph["metadata"]["multiple_bar_offset"] = 8;
$graph["metadata"]["show_x_labels"] = true;
$graph["metadata"]["show_legend"] = 1;
$graph["metadata"]["show_gridlines"] = 4;
$graph["metadata"]["show_proctime"] = 1;
$graph["metadata"]["show_gentime"] = 1;
$graph["metadata"]["x_label_angle"] = 45;

echo ("<p>");
draw_graph($graph);
echo ("</p>");

$graph = array();

$array = array();
$array["data"] = array(1 => 5, 2 => 10, 4 => 15, 8 => 20, 16 => 25, 32 => 30);
$array["colour"] = array("r" => 202, "g" => 167, "b" => 22);
$array["legend"] = "Test1";
$graph[] = $array;

$array = array();
$array["data"] = array(1 => 75, 20 => 75, 10 => 25, 32 => 25);
$array["colour"] = array("r" => 245, "g" => 141, "b" => 124);
$array["legend"] = "Z Test";
$graph[] = $array;

$graph["metadata"]["title"] = "Two Plots";
$graph["metadata"]["bgcolour"] = array("r" => 200, "g" => 200, "b" => 200);
$graph["metadata"]["x_axis_type"] = "scaled";
$graph["metadata"]["xsize"] = 500;
$graph["metadata"]["ysize"] = 300;
$graph["metadata"]["ymin"] = 0;
$graph["metadata"]["ymax"] = 100;
$graph["metadata"]["show_x_labels"] = true;
$graph["metadata"]["show_legend"] = 1;
$graph["metadata"]["show_gridlines"] = 4;
$graph["metadata"]["show_proctime"] = 1;
$graph["metadata"]["show_gentime"] = 1;

echo ("<p>");
draw_graph($graph);
echo ("</p>");

?>
